using UnityEngine;
using System.Collections;

public class CarMovementController : MonoBehaviour
{
	
	// Acceleration variables
	public float lastAccelerometerX;
	public float baseAccelerometerX;
	
	public float acceleration;
	public float maxAcceleration = 200f;
	public float maxReverseAcceleration = -40f;

	public float speed;
	
	// Maximum rigidbody speed 
	// when multipled by 200, it gives real in-game car speed
	public float maxVelocity = 1;

	public bool controlsEnabled;

	private Vector3 _calibrationBuffer;
	private Vector3 _rotationVector;

	private bool _calibrate;

	void Start ()
	{
		_calibrate = true;
		lastAccelerometerX = Config.getAccelerometer (Input.acceleration).x;
		baseAccelerometerX = lastAccelerometerX;
		
		//Disable controls until GO! animation fades
		controlsEnabled = false;
		
		// Set basic acceleration value
		acceleration = 0;
		
		// Adjust center of mass point
		rigidbody.centerOfMass = rigidbody.centerOfMass - new Vector3 (0f, 0f, -0.01f);
	}

	void Update ()
	{
		speed = Mathf.Floor (rigidbody.velocity.magnitude * 200);
		
		// Calibrate
		if (_calibrate) {
			Config.calibrateAccelerometer ();
			_calibrate = false;
		}
		
		// Car accelerometer movement controls
		// ..but first check "enabled" switch
		if (controlsEnabled) {
			CheckForRotation ();
			CheckForMovement ();
		} else {
			// Decay acceleration
			if (Mathf.Abs(acceleration) != 0){
				Mathf.Lerp(acceleration, 0, 10 * Time.deltaTime);
				if (acceleration < 10){
					acceleration = 0;
				}
			}
		}
	}

	void CheckForRotation ()
	{
		_rotationVector = new Vector3 (0, 0, 0);
		// Get car rotation data
		if (rigidbody.velocity.magnitude > 0.05f) {
			if (Input.GetKey(KeyCode.LeftArrow)) {
				_rotationVector.y = -0.5f;
			} else if (Input.GetKey(KeyCode.RightArrow)) {
				_rotationVector.y = 0.5f;
			} else {
				_rotationVector.y = -Config.getAccelerometer (Input.acceleration).y;
			}
		} 
		Quaternion deltaRotation = Quaternion.Euler (_rotationVector * 1.5f);
		rigidbody.MoveRotation (rigidbody.rotation * deltaRotation);
	}

	void CheckForMovement ()
	{
		
		// Check for car accelerometer acceleration..
		if (Config.getAccelerometer (Input.acceleration).x > (baseAccelerometerX + 0.06f) && acceleration < maxAcceleration) {
			lastAccelerometerX = Config.getAccelerometer (Input.acceleration).x;
			if (acceleration < maxAcceleration) {
				// Accelerate!
				acceleration = (acceleration + 3.5f * Mathf.Abs (lastAccelerometerX));
			} else {
				acceleration = maxAcceleration;
			}
			// or check if it should slow down	
		} else if (Config.getAccelerometer (Input.acceleration).x < (baseAccelerometerX - 0.06f) && acceleration > maxReverseAcceleration) {
			lastAccelerometerX = Config.getAccelerometer (Input.acceleration).x;
			if (acceleration >= maxReverseAcceleration) {
				// Slow down!
				acceleration = (acceleration - 4.3f * Mathf.Abs (lastAccelerometerX));
			} else {
				acceleration = maxReverseAcceleration;
			}
		}

		// And here's some basic support for keyboard controllers
		if (Input.GetKey(KeyCode.UpArrow)) {
			acceleration += 1;
			if (acceleration >= maxAcceleration) {
				acceleration = maxAcceleration;
			}
		} else if (Input.GetKey(KeyCode.DownArrow)) {
			acceleration -= 3;
			if (acceleration < maxReverseAcceleration) {
				acceleration = maxReverseAcceleration;
			}
		}

		// Push!
		rigidbody.AddForce (transform.TransformDirection (Vector3.forward * acceleration * 1.35f * Time.deltaTime));
	}
}
