using UnityEngine;
using System.Collections;

public class CarSoundController : MonoBehaviour
{
	private CarMovementController _carScript;

	public AudioClip D;
	public float DVolume = 1.0f;
	public AudioClip E;
	public float EVolume = 1.0f;
	public AudioClip F;
	public float FVolume = 1.0f;
	public AudioClip K;
	public float KVolume = 1.0f;
	public AudioClip L;
	public float LVolume = 1.0f;

	public AudioClip crashLowSpeedSound;
	public float crashLowVolume = 1.0f;
	public AudioClip crashHighSpeedSound;
	public float crashHighVolume = 1;

	public float gearShiftTime = 0.1f;
	public bool shiftingGear = false;
	public int gearShiftsStarted = 0;

	private int _crashesStarted = 0;
	private float _crashTime = 0.2f;
	private int _oneShotLimit = 8;

	public float idleFadeStartSpeed = 3.0f;
	public float idleFadeStopSpeed = 10.0f;
	public float idleFadeSpeedDiff = 7.0f;
	public float speedFadeStartSpeed = 0;
	public float speedFadeStopSpeed = 8.0f;
	public float speedFadeSpeedDiff = 8.0f;

	public bool soundsSet = false;

	public float inputFactor = 0;
	public int gear = 0;
	public int topGear = 6;

	public float idlePitch = 0.7f;
	public float startPitch = 0.85f;
	public float lowPitch = 1.17f;
	public float medPitch = 1.25f;
	public float highPitchFirst = 1.65f;
	public float highPitchSecond = 1.76f;
	public float highPitchThird = 1.80f;
	public float highPitchFourth = 1.86f;
	public float shiftPitch = 1.44f;

	private float prevPitchFactor = 0;

	private AudioSource _DAudio;
	private AudioSource _EAudio;
	private AudioSource _FAudio;
	private AudioSource _KAudio;
	private AudioSource _LAudio;
	private AudioSource _carAudio;

	// Use this for initialization
	void Awake ()
	{
		DVolume *= 0.5f;
		EVolume *= 0.5f;
		FVolume *= 0.2f;
		KVolume *= 0.7f;
		LVolume *= 0.5f;
		
		_DAudio = gameObject.AddComponent<AudioSource> ();
		_DAudio.loop = true;
		_DAudio.playOnAwake = false;
		_DAudio.clip = D;
		_DAudio.volume = DVolume;
		
		_EAudio = gameObject.AddComponent<AudioSource> ();
		_EAudio.loop = true;
		_EAudio.playOnAwake = false;
		_EAudio.clip = E;
		_EAudio.volume = EVolume;
		
		_FAudio = gameObject.AddComponent<AudioSource> ();
		_FAudio.loop = true;
		_FAudio.playOnAwake = false;
		_FAudio.clip = F;
		_FAudio.volume = FVolume;
		
		_KAudio = gameObject.AddComponent<AudioSource> ();
		_KAudio.loop = true;
		_KAudio.playOnAwake = false;
		_KAudio.clip = K;
		_KAudio.volume = KVolume;
		
		_LAudio = gameObject.AddComponent<AudioSource> ();
		_LAudio.loop = true;
		_LAudio.playOnAwake = false;
		_LAudio.clip = L;
		_LAudio.volume = LVolume;
		
		_carAudio = gameObject.AddComponent<AudioSource>();
		_carAudio.loop = false;
		_carAudio.playOnAwake = false;
		_carAudio.Stop();

		_DAudio.Play();
		_EAudio.Play();
		_FAudio.Play();
		_KAudio.Play();
		_LAudio.Play();

		_carScript = gameObject.GetComponent<CarMovementController>();
		_crashTime = Mathf.Max (crashLowSpeedSound.length, crashHighSpeedSound.length);
	}

	// Update is called once per frame
	void Update ()
	{
		float carSpeed = gameObject.rigidbody.velocity.magnitude;
		float carSpeedFactor = Mathf.Clamp01 (carSpeed / _carScript.maxVelocity);
		
		_KAudio.volume = Mathf.Lerp (0, KVolume, carSpeedFactor);
		
		if (shiftingGear)
			return;
		
		float pitchFactor = Sinerp (0, topGear, carSpeedFactor);
		int newGear = (int)pitchFactor;
		
		pitchFactor -= newGear;
		float throttleFactor = pitchFactor;
		pitchFactor *= 0.3f;
		pitchFactor += throttleFactor * (0.7f) * Mathf.Clamp01 (Config.getAccelerometer (Input.acceleration).x * 2);
		
		if (newGear != gear) {
			if (newGear > gear)
				GearShift (prevPitchFactor, true);
			else
				GearShift (prevPitchFactor, false);
			gear = newGear;
		} else {
			float newPitch = 0;
			if (gear == 0)
				newPitch = Mathf.Lerp (idlePitch, highPitchFirst, pitchFactor); else if (gear == 1)
				newPitch = Mathf.Lerp (startPitch, highPitchSecond, pitchFactor); else if (gear == 2)
				newPitch = Mathf.Lerp (lowPitch, highPitchThird, pitchFactor);
			else
				newPitch = Mathf.Lerp (medPitch, highPitchFourth, pitchFactor);
			SetPitch (newPitch);
			SetVolume (newPitch);
		}
		prevPitchFactor = pitchFactor;
	}

	public float Coserp (float start, float end, float value)
	{
		return Mathf.Lerp (start, end, 1.0f - Mathf.Cos (value * Mathf.PI * 0.5f));
	}

	public float Sinerp (float start, float end, float value)
	{
		return Mathf.Lerp (start, end, Mathf.Sin (value * Mathf.PI * 0.5f));
	}

	public void GearShift (float oldPitchFactor, bool shiftUp)
	{
		shiftingGear = true;
		
		float timer = 0;
		float pitchFactor = 0;
		float newPitch = 0;
		
		if (shiftUp) {
			StartCoroutine (ShiftUp (timer, pitchFactor, newPitch, oldPitchFactor));
		} else {
			StartCoroutine (ShiftDown (timer, pitchFactor, newPitch));
		}
		
		shiftingGear = false;
	}

	IEnumerator ShiftUp (float timer, float pitchFactor, float newPitch, float oldPitchFactor)
	{
		while (timer < gearShiftTime) {
			pitchFactor = Mathf.Lerp (oldPitchFactor, 0, timer / gearShiftTime);
			if (gear == 0)
				newPitch = Mathf.Lerp (lowPitch, highPitchFirst, pitchFactor);
			else
				newPitch = Mathf.Lerp (lowPitch, highPitchSecond, pitchFactor);
			SetPitch (newPitch);
			SetVolume (newPitch);
			timer += Time.deltaTime;
			yield return null;
		}
	}

	IEnumerator ShiftDown (float timer, float pitchFactor, float newPitch)
	{
		while (timer < gearShiftTime) {
			pitchFactor = Mathf.Lerp (0, 1, timer / gearShiftTime);
			newPitch = Mathf.Lerp (lowPitch, shiftPitch, pitchFactor);
			SetPitch (newPitch);
			SetVolume (newPitch);
			timer += Time.deltaTime;
			yield return null;
		}
	}

	public void SetPitch (float pitch)
	{
		_DAudio.pitch = pitch;
		_EAudio.pitch = pitch;
		_FAudio.pitch = pitch;
		_LAudio.pitch = pitch;
	}

	public void SetVolume (float pitch)
	{
		float pitchFactor = Mathf.Lerp (0, 1f, (pitch - startPitch) / (highPitchSecond - startPitch));
		_DAudio.volume = Mathf.Lerp (0, DVolume, pitchFactor);
		float fVolume = Mathf.Lerp (FVolume * 0.80f, FVolume, pitchFactor);
		_FAudio.volume = fVolume * 0.7f + fVolume * 0.3f * Mathf.Clamp01 (Config.getAccelerometer (Input.acceleration).x);
		float eVolume = Mathf.Lerp (EVolume * 0.89f, EVolume, pitchFactor);
		_EAudio.volume = eVolume * 0.8f + eVolume * 0.2f * Mathf.Clamp01 (Config.getAccelerometer (Input.acceleration).x);
	}

	public IEnumerator Crash (float volumeFactor)
	{
		if (_crashesStarted > 3 || OneShotLimitReached ())
			yield return null;
		if (volumeFactor > 0.9f)
			_carAudio.PlayOneShot (crashHighSpeedSound, Mathf.Clamp01 ((0.5f + volumeFactor * 0.5f) * crashHighVolume));
		_carAudio.PlayOneShot (crashLowSpeedSound, Mathf.Clamp01 (volumeFactor * crashLowVolume));
		_crashesStarted++;
		
		yield return new WaitForSeconds (_crashTime);
		
		_crashesStarted--;
	}

	public bool OneShotLimitReached ()
	{
		return ((_crashesStarted + gearShiftsStarted) > _oneShotLimit) ? true : false;
	}
	
}

