using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class SimpleAnimator : MonoBehaviour
{
	// Used for common object animation
	private GameObject[] _gameObjectList;
	private VectorSet[] _vectorSetList;

	private bool _animateGroup;
	private bool _animateForward;
	private bool _enable;
	private float _animationSpeed;
	private int _animationsFinished;
	// --------------------------------
	
	// Scaling variable
	private bool _scaleGroup;
	
	// --------------------------------
	
	// Static coordinates used by different animation calls
	public const float OFFSCREENRIGHT = 950;
	public const float OFFSCREENLEFT = -905;
	public const float OFFSCREENTOP = -700;
	public const float OFFSCREENBOTTOM = 700;


	// Use this for initialization
	void Awake ()
	{
		_animateGroup = false;
		_scaleGroup = false;
	}

	// Update is called once per frame
	void Update ()
	{
		if (_animateGroup) {
			
			switch (_animateForward) {
			
			case true:
				for (int i = 0; i < _gameObjectList.Length; i++) {
					if (_vectorSetList[i].isActive ()) {
						
						if (_enable && _gameObjectList[i].active == false) {
							_gameObjectList[i].active = true;
						}
						
						_vectorSetList[i].setCurrentPosition (Vector3.Lerp (_vectorSetList[i].getCurrentPosition (), _vectorSetList[i].getEndPosition (), _animationSpeed * Time.deltaTime));
						if (Vector3.Distance (_gameObjectList[i].transform.position, _vectorSetList[i].getEndPosition ()) <= 0.08f) {
							_gameObjectList[i].transform.position = _vectorSetList[i].getEndPosition ();
							_vectorSetList[i].setInActive ();
							
							if (!_enable) {
								_gameObjectList[i].active = false;
							}
							
							_animationsFinished++;
						} else {
							_gameObjectList[i].transform.position = _vectorSetList[i].getCurrentPosition ();
						}
					}
				}
				
				break;
			
			case false:
				for (int i = 0; i < _gameObjectList.Length; i++) {
					if (_vectorSetList[i].isActive ()) {
						
						if (_enable && _gameObjectList[i].active == false) {
							_gameObjectList[i].active = true;
						}
						
						_vectorSetList[i].setCurrentPosition (Vector3.Lerp (_vectorSetList[i].getCurrentPosition (), _vectorSetList[i].getStartPosition (), _animationSpeed * Time.deltaTime));
						if (Vector3.Distance (_gameObjectList[i].transform.position, _vectorSetList[i].getStartPosition ()) <= 0.08f) {
							_gameObjectList[i].transform.position = _vectorSetList[i].getStartPosition ();
							_vectorSetList[i].setInActive ();
							
							if (!_enable) {
								_gameObjectList[i].active = false;
							}
							
							_animationsFinished++;
						} else {
							_gameObjectList[i].transform.position = _vectorSetList[i].getCurrentPosition ();
						}
					}
				}
				
				break;
				
			}
			if (_animationsFinished == _gameObjectList.Length && _animateGroup == true) {
				_animationsFinished = 0;
				_animateGroup = false;
			}
			
		}
	}

	public void AnimateGroup (VectorSet[] vectorList, GameObject[] gameObjectList, float animationSpeed, bool animateForward, bool enable)
	{
		if (!_animateGroup) {
			_animateForward = animateForward;
			_animationSpeed = animationSpeed;
			_vectorSetList = vectorList;
			_gameObjectList = gameObjectList;
			_enable = enable;
			_animationsFinished = 0;
			_animateGroup = true;
		}
	}

	
	public bool IsAnimationComencing(){
		return _animateGroup;
	}

}
